"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WfsDataParser = void 0;
var isEmpty = require('lodash/isEmpty');
var isFinite = require('lodash/isFinite');
var fast_xml_parser_1 = require("fast-xml-parser");
/**
 * Class implementing DataParser to fetch schema and sample data
 * using WFS requests (DescribeFeatureType resp. GetFeature)
 */
var WfsDataParser = /** @class */ (function () {
    function WfsDataParser() {
        this.title = 'WFS Data Parser';
    }
    /**
     * Generate request parameter string for a passed object
     *
     * @param {Object} params Object holding request params
     * @return {string} The URI encoded request parameter string joined with &
     */
    WfsDataParser.prototype.generateRequestParamString = function (params) {
        return Object.keys(params)
            .filter(function (key) { return !isEmpty(params[key]) || isFinite(params[key]); })
            .map(function (key) { return encodeURIComponent(key) + '=' + encodeURIComponent(params[key]); })
            .join('&');
    };
    /**
     * Map XSD datatypes (SimpleType) to json schema data stypes
     *
     * @param qualifiedXsdDataType The XSD datatype as string (including namespace)
     * @returns {JSONSchema4TypeName} The corresponding JSON schema type name
     */
    WfsDataParser.prototype.mapXsdTypeToJsonDataType = function (qualifiedXsdDataType) {
        var xsdDataType = qualifiedXsdDataType.indexOf(':') > -1
            ? qualifiedXsdDataType.split(':')[1]
            : qualifiedXsdDataType;
        switch (xsdDataType) {
            case 'string':
                return 'string';
            case 'boolean':
                return 'boolean';
            case 'float':
            case 'double':
            case 'long':
            case 'byte':
            case 'decimal':
            case 'integer':
            case 'int':
            case 'positiveInteger':
            case 'negativeInteger':
            case 'nonPositiveInteger':
            case 'nonNegativeInteger':
            case 'short':
            case 'unsignedLong':
            case 'unsignedInt':
            case 'unsignedShort':
            case 'unsignedByte':
                return 'number';
            default:
                return 'string';
        }
    };
    /**
     * Fetch schema and sample data and transforms it to the GeoStyler data model.
     *
     * Currently, WFS service must support application/json as outputFormat
     * and needs CORS headers (only needed if WFS Service is not located on the same origin
     * as the component using this parser) to be available in responses
     *
     * @param wfsConfig The parameters of the WFS
     */
    WfsDataParser.prototype.readData = function (_a) {
        var _this = this;
        var url = _a.url, requestParams = _a.requestParams, _b = _a.fetchParams, fetchParams = _b === void 0 ? {} : _b;
        var desribeFeatureTypeOpts;
        if (requestParams.version === '1.1.0') {
            desribeFeatureTypeOpts = {
                version: requestParams.version,
                maxFeatures: requestParams.maxFeatures,
                typeName: requestParams.typeName
            };
        }
        else {
            desribeFeatureTypeOpts = {
                version: requestParams.version,
                count: requestParams.count,
                typeNames: requestParams.typeNames
            };
        }
        var describeFeatureTypeParams = __assign(__assign({}, desribeFeatureTypeOpts), { service: 'WFS', request: 'DescribeFeatureType' });
        var getFeatureParams = __assign(__assign({}, requestParams), { service: 'WFS', request: 'GetFeature', outputFormat: 'application/json' });
        // Fetch data schema via describe feature type
        var requestDescribeFeatureType = "".concat(url, "?").concat(this.generateRequestParamString(describeFeatureTypeParams));
        var describeFeatureTypePromise = new Promise(function (resolve, reject) {
            fetch(requestDescribeFeatureType, fetchParams)
                .then(function (response) { return response.text(); })
                .then(function (describeFeatueTypeResult) {
                var _a, _b, _c, _d, _e;
                try {
                    var parser = new fast_xml_parser_1.XMLParser({
                        removeNSPrefix: true,
                        ignoreDeclaration: true,
                        ignoreAttributes: false
                    });
                    var result = parser.parse(describeFeatueTypeResult);
                    var attributes = (_e = (_d = (_c = (_b = (_a = result === null || result === void 0 ? void 0 : result.schema) === null || _a === void 0 ? void 0 : _a.complexType) === null || _b === void 0 ? void 0 : _b.complexContent) === null || _c === void 0 ? void 0 : _c.extension) === null || _d === void 0 ? void 0 : _d.sequence) === null || _e === void 0 ? void 0 : _e.element;
                    var properties_1 = {};
                    attributes.forEach(function (attr) {
                        var name = attr['@_name'];
                        var type = attr['@_type'];
                        if (!properties_1[name]) {
                            var propertyType = { type: _this.mapXsdTypeToJsonDataType(type) };
                            properties_1[name] = propertyType;
                        }
                    });
                    var title = result.schema.element['@_name'];
                    var schema = {
                        type: 'object',
                        title: title,
                        properties: properties_1
                    };
                    resolve(schema);
                }
                catch (error) {
                    reject("Error while parsing DescribeFeatureType response: ".concat(error));
                }
            })
                .catch(function (error) { return reject("Could not parse XML document: ".concat(error)); });
        });
        // Fetch sample data via WFS GetFeature
        var requestGetFeature = "".concat(url, "?").concat(this.generateRequestParamString(getFeatureParams));
        var getFeaturePromise = new Promise(function (resolve, reject) {
            fetch(requestGetFeature, fetchParams)
                .then(function (response) { return response.json(); })
                .then(function (getFeatureResult) {
                var fc = getFeatureResult;
                resolve(fc);
            })
                .catch(function (err) {
                var emptyFc = {
                    type: 'FeatureCollection',
                    features: []
                };
                reject(emptyFc);
            });
        });
        return new Promise(function (resolve, reject) {
            // Fetch features and type definition in parallel and
            // resolve if both are available
            Promise.all([describeFeatureTypePromise, getFeaturePromise])
                .then(function (results) {
                var schema = results[0], exampleFeatures = results[1];
                var data = {
                    schema: schema,
                    exampleFeatures: exampleFeatures
                };
                resolve(data);
            })
                .catch(function (error) { return reject(error); });
        });
    };
    /**
     * The name of the WfsDataParser.
     */
    WfsDataParser.title = 'WFS Data Parser';
    return WfsDataParser;
}());
exports.WfsDataParser = WfsDataParser;
exports.default = WfsDataParser;
//# sourceMappingURL=WfsDataParser.js.map